﻿// ***************************************************************
// Reads the XML and returns agent configuration.
// 
// ***************************************************************

namespace routeBalancer
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// This class stores settings that will be
    /// used by the agent
    /// </summary>
    public class RouteBalancerSettings
    {
        #region Fields


        /// <summary>
        /// The number of connectors to use.
        /// These send connectors must be created on Exchange 2010 with diferent address spaces
        /// like connectorN.local, where N is the number of connector begining at 1.
        /// </summary>
        private int numConnectors;


        #endregion Fields


        #region Constructors
        /// <summary>
        /// Empty constructor initializes with default values.
        /// </summary>
        /// <param name="filePath">The path to an XML file containing the settings</param>
        public RouteBalancerSettings(string filePath)
        {
            // Default : 1 connector
            this.numConnectors = 1;

            // Read non-default settings from file
            this.ReadXMLConfig(filePath);
        }


        #endregion Constructors


        #region Parameters
        /// <summary>
        /// Get the number of connectors to use.
        /// </summary>
        public int NumConnectors
        {
            get { return this.numConnectors; }
                       
        }

        #endregion Parameters


        #region XML File Parsing
        /// <summary>
        /// Reads in configuration options from an XML file and sets the instance
        /// variables to the corresponding values that are read in if they
        /// are valid.  If an invalid value is found, or a value is not
        /// set in the XML file, the variable will not be changed from its
        /// default value;
        /// </summary>
        /// <param name="path">The path to the XML configuration file</param>
        /// <returns>True if the file was read.</returns>
        public bool ReadXMLConfig(string path)
        {
            bool retval = false;

            try
            {
                // Some temp variables that will be used during validation
                
                int fileInt = 0;

                // Load the file into the XML reader
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(path);

                XmlNode xmlRoot = xmlDoc.SelectSingleNode("RouteBalancerSettings");

               
                // Reads the number of connectors to use
                fileInt = this.ReadXmlInt(xmlRoot, "numConnectors");
                if (fileInt > 0)
                {
                    this.numConnectors = fileInt;
                }

            }

            #region XML and File IO Catches
            catch (XmlException e)
            {
                Debug.WriteLine(e.ToString());
                return false;
            }
            catch (UnauthorizedAccessException e)
            {
                Debug.WriteLine(e.ToString());
                return false;
            }
            catch (IOException e)
            {
                Debug.WriteLine(e.ToString());
                return false;
            }
            #endregion XML File IO Catches
            return retval;
        }

        /// <summary>
        /// Reads an integer value based on the name of the element it is the value of
        /// </summary>
        /// <param name="root">The root element to start searching from</param>
        /// <param name="xmlParam">The name of the element to get the value of</param>
        /// <returns>The parsed value, or 0 if it was not found.</returns>
        private int ReadXmlInt(XmlNode root, string xmlParam)
        {
            int retval = 0;

            if (root != null && xmlParam != null)
            {
                XmlNode valNode = root.SelectSingleNode(xmlParam);
                if (valNode != null)
                {
                    XmlNode childNode = valNode.FirstChild;
                    if (childNode != null)
                    {
                        int.TryParse(childNode.Value, out retval);
                    }
                }
            }

            return retval;
        }

        #endregion XML File Parsing
    }
}
