﻿using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Reflection;

using Microsoft.Exchange.Data.Transport;
using Microsoft.Exchange.Data.Transport.Email;
using Microsoft.Exchange.Data.Transport.Smtp;
using Microsoft.Exchange.Data.Transport.Routing;

namespace routeBalancer
{
    public class routeBalancerAgentFactory : RoutingAgentFactory
    {


        /// <summary>
        /// The path where data files are stored relative to
        /// the DLL containing this code.
        /// </summary>
        private const string RelativeDataPath = @"data\";

        /// <summary>
        /// The filename of the XML file containing configuration
        /// data for the routeBalancer Agent.
        /// </summary>
        private const string ConfigFileName = "RouteBalancerConfig.xml";


        /// <summary>
        /// The filename of the LOG file.
        /// </summary>
        private const string LogFileName = "RouteBalancerAgentLog.txt";

        /// <summary>
        /// An object containing settings to be used by the agent.
        /// </summary>
        public RouteBalancerSettings routeBalancerSettings;

        /// <summary>
        /// The location on disk where data files such as
        /// the configuration file and log are stored.
        /// The value will be calculated relative to the
        /// location of the DLL containing the assembly
        /// of this class.
        /// </summary>
        private string agentPath;
        private string logPath;


        /// <summary>
        /// The number of connectors (routingOverrides)
        /// to be used to balance.
        /// An index to round robin the connector for each message.
        /// </summary>
        private int iNumConnectors;
        private RoutingOverride[] connectors;
        private int iIndexConnector = 0;
              
        #region Constructor

        public routeBalancerAgentFactory()
        {
            Assembly currAssembly = Assembly.GetAssembly(this.GetType());
            this.agentPath = Path.GetDirectoryName(currAssembly.Location);
            this.routeBalancerSettings = new RouteBalancerSettings(Path.Combine(this.agentPath, ConfigFileName));
            this.logPath = Path.Combine(agentPath, LogFileName);

            iNumConnectors = this.routeBalancerSettings.NumConnectors;
            connectors = new RoutingOverride[iNumConnectors];


            //loads the array with the number of routingOverrides defined on config file
            for (int i = 1; i <= iNumConnectors; i++)
            {
                //Defining routing domain (this would select the connector 
                //to go based on its dummy address space i.e. connector1.local)
                string connectorDomain = "connector" + i + ".local";
                RoutingDomain myRoutingDomain = new RoutingDomain(connectorDomain);

                //Defining routing override and setting "UseRecipientDomain" so the connector
                //will resolve the recipient's MX domain instead of the override domain
                //This would allow to select the connector without modifying the next hop original destination
                connectors[i-1] = (new RoutingOverride(myRoutingDomain, DeliveryQueueDomain.UseRecipientDomain));
               
            }
 
       }
        #endregion Constructor

        public override void Close()
        { }

        public override RoutingAgent CreateAgent(SmtpServer server)
        {
            return new routeBalancerAgent(this.getNextConnector());
        }

        private RoutingOverride getNextConnector()
        {
            RoutingOverride tmpConnector;

            if (connectors.Length == iIndexConnector)
            {
                iIndexConnector = 0;
            }

            tmpConnector = (RoutingOverride)connectors[iIndexConnector];
            iIndexConnector = iIndexConnector+1;
            return tmpConnector;
         }
       
    }
}
